@@ -33,17 +33,16 @@ References:
 
 import math
 from abc import abstractmethod
-from typing import Any, Dict, List, Mapping, Optional, Tuple, Union
+from typing import Any, Dict, List, Mapping, Optional, Tuple
 
 import pyro
 import torch
 from botorch.acquisition.objective import PosteriorTransform
-from botorch.models.gp_regression import FixedNoiseGP, SingleTaskGP
+from botorch.models.gpytorch import BatchedMultiOutputGPyTorchModel
 from botorch.models.transforms.input import InputTransform
 from botorch.models.transforms.outcome import OutcomeTransform
 from botorch.models.utils import validate_input_scaling
 from botorch.posteriors.fully_bayesian import FullyBayesianPosterior, MCMC_DIM
-from botorch.sampling.samplers import MCSampler
 from gpytorch.constraints import GreaterThan
 from gpytorch.distributions.multivariate_normal import MultivariateNormal
 from gpytorch.kernels import MaternKernel, ScaleKernel
@@ -55,6 +54,7 @@ from gpytorch.likelihoods.gaussian_likelihood import (
 from gpytorch.likelihoods.likelihood import Likelihood
 from gpytorch.means.constant_mean import ConstantMean
 from gpytorch.means.mean import Mean
+from gpytorch.models.exact_gp import ExactGP
 from torch import Tensor
 
 MIN_INFERRED_NOISE_LEVEL = 1e-6
@@ -295,7 +295,7 @@ class SaasPyroModel(PyroModel):
         return mean_module, covar_module, likelihood
 
 
-class SaasFullyBayesianSingleTaskGP(SingleTaskGP):
+class SaasFullyBayesianSingleTaskGP(ExactGP, BatchedMultiOutputGPyTorchModel):
     r"""A fully Bayesian single-task GP model with the SAAS prior.
 
     This model assumes that the inputs have been normalized to [0, 1]^d and that
@@ -359,11 +359,15 @@ class SaasFullyBayesianSingleTaskGP(SingleTaskGP):
         validate_input_scaling(
             train_X=transformed_X, train_Y=train_Y, train_Yvar=train_Yvar
         )
-        self._set_dimensions(train_X=train_X, train_Y=train_Y)
+        self._num_outputs = train_Y.shape[-1]
+        self._input_batch_shape = train_X.shape[:-2]
         if train_Yvar is not None:  # Clamp after transforming
             train_Yvar = train_Yvar.clamp(MIN_INFERRED_NOISE_LEVEL)
 
-        super().__init__(train_X, train_Y)
+        X_tf, Y_tf, _ = self._transform_tensor_args(X=train_X, Y=train_Y)
+        super().__init__(
+            train_inputs=X_tf, train_targets=Y_tf, likelihood=GaussianLikelihood()
+        )
         self.mean_module = None
         self.covar_module = None
         self.likelihood = None
@@ -399,14 +403,20 @@ class SaasFullyBayesianSingleTaskGP(SingleTaskGP):
         self._check_if_fitted()
         return len(self.covar_module.outputscale)
 
-    def fantasize(
-        self,
-        X: Tensor,
-        sampler: MCSampler,
-        observation_noise: Union[bool, Tensor] = True,
-        **kwargs: Any,
-    ) -> FixedNoiseGP:
-        raise NotImplementedError("Fantasize is not implemented!")
+    @property
+    def batch_shape(self) -> torch.Size:
+        r"""Batch shape of the model, equal to the number of MCMC samples.
+        Note that `SaasFullyBayesianSingleTaskGP` does not support batching
+        over input data at this point."""
+        return torch.Size([self.num_mcmc_samples])
+
+    @property
+    def _aug_batch_shape(self) -> torch.Size:
+        r"""The batch shape of the model, augmented to include the output dim."""
+        aug_batch_shape = self.batch_shape
+        if self.num_outputs > 1:
+            aug_batch_shape += torch.Size([self.num_outputs])
+        return aug_batch_shape
 
     def train(self, mode: bool = True) -> None:
         r"""Puts the model in `train` mode."""
@@ -464,9 +474,19 @@ class SaasFullyBayesianSingleTaskGP(SingleTaskGP):
         super().load_state_dict(state_dict=state_dict, strict=strict)
 
     def forward(self, X: Tensor) -> MultivariateNormal:
+        """
+        Unlike in other classes' `forward` methods, there is no `if self.training`
+        block, because it ought to be unreachable: If `self.train()` has been called,
+        then `self.covar_module` will be None, `check_if_fitted()` will fail, and the
+        rest of this method will not run.
+        """
         self._check_if_fitted()
-        return super().forward(X.unsqueeze(MCMC_DIM))
+        x = X.unsqueeze(MCMC_DIM)
+        mean_x = self.mean_module(x)
+        covar_x = self.covar_module(x)
+        return MultivariateNormal(mean_x, covar_x)
 
+    # pyre-ignore[14]: Inconsistent override
     def posterior(
         self,
         X: Tensor,
