@@ -8,7 +8,7 @@ r"""Multi-task Gaussian Process Regression models with fully Bayesian inference.
 """
 
 
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Any, Dict, List, NoReturn, Optional, Tuple
 
 import pyro
 import torch
@@ -24,7 +24,6 @@ from botorch.models.multitask import FixedNoiseMultiTaskGP
 from botorch.models.transforms.input import InputTransform
 from botorch.models.transforms.outcome import OutcomeTransform
 from botorch.posteriors.fully_bayesian import FullyBayesianPosterior, MCMC_DIM
-from botorch.sampling.samplers import MCSampler
 from botorch.utils.datasets import SupervisedDataset
 from gpytorch.distributions.multivariate_normal import MultivariateNormal
 from gpytorch.kernels import MaternKernel
@@ -292,13 +291,15 @@ class SaasFullyBayesianMultiTaskGP(FixedNoiseMultiTaskGP):
         self._check_if_fitted()
         return len(self.covar_module.outputscale)
 
-    def fantasize(
-        self,
-        X: Tensor,
-        sampler: MCSampler,
-        observation_noise: Union[bool, Tensor] = True,
-        **kwargs: Any,
-    ) -> FixedNoiseMultiTaskGP:
+    @property
+    def batch_shape(self) -> torch.Size:
+        r"""Batch shape of the model, equal to the number of MCMC samples.
+        Note that `SaasFullyBayesianMultiTaskGP` does not support batching
+        over input data at this point."""
+        self._check_if_fitted()
+        return torch.Size([self.num_mcmc_samples])
+
+    def fantasize(self, *args, **kwargs) -> NoReturn:
         raise NotImplementedError("Fantasize is not implemented!")
 
     def _check_if_fitted(self):
@@ -323,6 +324,8 @@ class SaasFullyBayesianMultiTaskGP(FixedNoiseMultiTaskGP):
             self.latent_features,
         ) = self.pyro_model.load_mcmc_samples(mcmc_samples=mcmc_samples)
 
+    # pyre-fixme[14]: Inconsistent override of
+    # BatchedMultiOutputGPyTorchModel.posterior
     def posterior(
         self,
         X: Tensor,
@@ -347,6 +350,7 @@ class SaasFullyBayesianMultiTaskGP(FixedNoiseMultiTaskGP):
         posterior = FullyBayesianPosterior(mvn=posterior.mvn)
         return posterior
 
+    # pyre-fixme[14]: Inconsistent override
     def forward(self, X: Tensor) -> MultivariateNormal:
         self._check_if_fitted()
         X = X.unsqueeze(MCMC_DIM)
@@ -375,6 +379,7 @@ class SaasFullyBayesianMultiTaskGP(FixedNoiseMultiTaskGP):
         return MultivariateNormal(mean_x, covar)
 
     @classmethod
+    # pyre-fixme[14]: Inconsistent override of `MultiTaskGP.construct_inputs`
     def construct_inputs(
         cls,
         training_data: Dict[str, SupervisedDataset],
