@@ -21,7 +21,7 @@ from typing import Any, Iterator, List, Optional, Tuple, Union
 
 import torch
 from botorch.acquisition.objective import PosteriorTransform
-from botorch.exceptions.errors import BotorchTensorDimensionError
+from botorch.exceptions.errors import BotorchTensorDimensionError, InputDataError
 from botorch.exceptions.warnings import BotorchTensorDimensionWarning
 from botorch.models.model import Model, ModelList
 from botorch.models.utils import (
@@ -53,6 +53,7 @@ class GPyTorchModel(Model, ABC):
         X: Tensor, Y: Tensor, Yvar: Optional[Tensor] = None, strict: bool = True
     ) -> None:
         r"""Checks that `Y` and `Yvar` have an explicit output dimension if strict.
+        Checks that the dtypes of the inputs match, and warns if using float.
 
         This also checks that `Yvar` has the same trailing dimensions as `Y`. Note
         we only infer that an explicit output dimension exists when `X` and `Y` have
@@ -100,6 +101,22 @@ class GPyTorchModel(Model, ABC):
                 f" Expected Yvar with shape: {Y.shape[-Yvar.dim() :]} (got"
                 f" {Yvar.shape})."
             )
+        # Check the dtypes.
+        if X.dtype != Y.dtype or (Yvar is not None and Y.dtype != Yvar.dtype):
+            raise InputDataError(
+                "Expected all inputs to share the same dtype. Got "
+                f"{X.dtype} for X, {Y.dtype} for Y, and "
+                f"{Yvar.dtype if Yvar is not None else None} for Yvar."
+            )
+        if X.dtype != torch.float64:
+            # NOTE: Not using a BotorchWarning since those get ignored.
+            warnings.warn(
+                f"The model inputs are of type {X.dtype}. It is strongly recommended "
+                "to use double precision in BoTorch, as this improves both "
+                "precision and stability and can help avoid numerical errors. "
+                "See https://github.com/pytorch/botorch/discussions/1444",
+                UserWarning,
+            )
 
     @property
     def batch_shape(self) -> torch.Size:
@@ -508,7 +525,7 @@ class ModelListGPyTorchModel(GPyTorchModel, ModelList, ABC):
         to the `posterior` method returns a Posterior object over an output of
         shape `broadcast(test_batch_shape, model.batch_shape) x q x m`.
         """
-        batch_shapes = {ti[0].shape[:-2] for ti in self.train_inputs}
+        batch_shapes = {m.batch_shape for m in self.models}
         if len(batch_shapes) > 1:
             msg = (
                 f"Component models of {self.__class__.__name__} have different "
@@ -562,7 +579,7 @@ class ModelListGPyTorchModel(GPyTorchModel, ModelList, ABC):
         with gpt_posterior_settings():
             # only compute what's necessary
             if output_indices is not None:
-                mvns = [self.forward_i(i, transformed_X[i]) for i in output_indices]
+                mvns = [self.models[i](transformed_X[i]) for i in output_indices]
                 if observation_noise is not False:
                     if torch.is_tensor(observation_noise):
                         lh_kwargs = [
